﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using CreditAgricole.Framework;

namespace CreditAgricole.Application
{
    public static class DataMep
    {
        #region MEP List Request

        /// <summary>
        /// Get all items in list
        /// </summary>
        /// <param name="siteId">Current site ID</param>
        /// <param name="webId">Current web ID</param>
        /// <param name="listTitle">List title where tasks were created</param>
        /// <returns>List of MepObj</returns>
        public static List<VOMep> GetMepObj(Guid siteId, Guid webId, string listTitle)
        {
            List<VOMep> mepObjs = new List<VOMep>();
            SPSecurity.RunWithElevatedPrivileges(delegate()
            {
                using (SPSite site = new SPSite(siteId))
                {
                    using (SPWeb web = site.OpenWeb(webId))
                    {
                        SPList list = web.Lists.TryGetList(listTitle);
                        if (list != null)
                        {
                            SPQuery query = QueryBuilder.GetSelectedItemsOrderBy(Constants.CreatedDateField);
                            SPListItemCollection items = list.GetItems(query);
                            AddItemsToMepList(mepObjs, items);
                        }
                    }
                }
            });
            return mepObjs;
        }

        /// <summary>
        /// Get all items created by current user
        /// </summary>
        /// <param name="siteId">Current site ID</param>
        /// <param name="webId">Current web ID</param>
        /// <param name="listTitle">List title where tasks were created</param>
        /// <returns>List of MepObj</returns>
        public static List<VOMep> GetCreatedTasksByWorkflowStatus(Guid siteId, Guid webId, string listTitle, 
                                                                    int currentUserId, 
                                                                    string workflowStatusNumber, 
                                                                    Constants.CamlQueryOperator queryOperator)
        {
            List<VOMep> mepObjs = new List<VOMep>();
            SPSecurity.RunWithElevatedPrivileges(delegate()
            {
                using (SPSite site = new SPSite(siteId))
                {
                    using (SPWeb web = site.OpenWeb(webId))
                    {
                        SPList list = web.Lists.TryGetList(listTitle);
                        if (list != null)
                        {
                            IDictionary<string, string> filter = new Dictionary<string, string>();
                            string workflowInternalName = SPHelper.GetInternalFieldName(list, Constants.WorkflowAssociationName);
                            if (!string.IsNullOrEmpty(workflowInternalName))
                            {
                                filter.Add(workflowInternalName, workflowStatusNumber);
                                SPQuery query = QueryBuilder.GetSelectedItemsWithLookup("Lookup", "Author", 
                                                                                         MappingTool.CatchNull(currentUserId), 
                                                                                         Constants.CamlQueryOperator.Equal, 
                                                                                         filter, queryOperator);
                                SPListItemCollection selectedItems = list.GetItems(query);
                                AddItemsToMepList(mepObjs, selectedItems);
                            }
                        }
                    }
                }
            });
            return mepObjs;
        }

        /// <summary>
        /// Get items with workflow status mentionned in parameter
        /// </summary>
        /// <param name="siteId">Current site ID</param>
        /// <param name="webId">Current web ID</param>
        /// <param name="listTitle">List title where tasks were created</param>
        /// <param name="status">Workflow status</param>
        /// <returns>List of MepObj</returns>
        public static List<VOMep> GetItemsByProdStatus(Guid siteId, Guid webId, string listTitle, string prodStatus, Constants.CamlQueryOperator prodStatusQueryOperator)
        {
            List<VOMep> mepObjs = new List<VOMep>();
            SPSecurity.RunWithElevatedPrivileges(delegate()
            {
                using (SPSite site = new SPSite(siteId))
                {
                    using (SPWeb web = site.OpenWeb(webId))
                    {
                        SPList list = web.Lists.TryGetList(listTitle);
                        if (list != null)
                        {
                            string workflowInternalName = SPHelper.GetInternalFieldName(list, Constants.WorkflowAssociationName);
                            if (!string.IsNullOrEmpty(workflowInternalName))
                            {
                                IDictionary<string, string> filter = new Dictionary<string, string>();
                                filter.Add(Constants.MepStatusField, prodStatus);
                                filter.Add(workflowInternalName, "16");
                                SPQuery query = QueryBuilder.GetSelectedItems(workflowInternalName, "16", Constants.CamlQueryOperator.Equal, Constants.MepStatusField, prodStatus, prodStatusQueryOperator);
                                SPListItemCollection selectedItems = list.GetItems(query);
                                AddItemsToMepList(mepObjs, selectedItems);
                            }
                        }
                    }
                }
            });
            return mepObjs;
        }

        /// <summary>
        /// Get the list of items awaiting to be validated by the current user
        /// </summary>
        /// <param name="siteId">The current site id</param>
        /// <param name="webId">The current web Id</param>
        /// <param name="listTitle"></param>
        /// <returns></returns>
        public static List<VOMep> GetTasksAssignedToMeByWorkflowStatus(Guid siteId, Guid webId, string listTitle, 
                                                                       int currentUserId, 
                                                                       Constants.CamlQueryOperator userQueryOperator, 
                                                                       string workflowStatusNumber, 
                                                                       Constants.CamlQueryOperator workflowQueryOperator)
        {
            List<VOMep> mepObjs = new List<VOMep>();
            SPSecurity.RunWithElevatedPrivileges(delegate()
            {
                using (SPSite site = new SPSite(siteId))
                {
                    using (SPWeb web = site.OpenWeb(webId))
                    {
                        SPList tasks = web.Lists.TryGetList(Constants.WorkflowTasksList);
                        SPList list = web.Lists.TryGetList(listTitle);
                        if (tasks != null && list != null)
                        {
                            IDictionary<string, string> filter = new Dictionary<string,string>();
                            filter.Add("WorkflowListId", MappingTool.CatchNull(list.ID));
                            SPQuery query = QueryBuilder.GetSelectedItemsWithLookup("User", "AssignedTo", MappingTool.CatchNull(currentUserId), userQueryOperator, filter, workflowQueryOperator); 
                            SPListItemCollection taskItems = tasks.GetItems(query);
                            string listPath = string.Format("{0}/{1}/", web.Url, list.RootFolder.Url);
                            SPListItemCollection listItems = GetTaskItemsFromList(list, taskItems, workflowStatusNumber, listPath);
                            AddItemsToMepList(mepObjs, listItems);
                        }
                    }
                }
            });
            return mepObjs;
        }

        #endregion

        #region Tool Methods

        /// <summary>
        /// Insert map all items and insert them in MepObjs list
        /// </summary>
        /// <param name="mepObjs">MepObjs List</param>
        /// <param name="items">Items to map</param>
        private static void AddItemsToMepList(List<VOMep> mepObjs, SPListItemCollection items)
        {
            try
            {
                foreach (SPListItem item in items)
                {
                    VOMep mepObj = GetMappedObj(item);
                    mepObjs.Add(mepObj);
                }
            }
            catch (SPException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.Message);
            }
        }

        /// <summary>
        /// Map object from List item
        /// </summary>
        /// <param name="item">List item</param>
        /// <returns>mapped object</returns>
        private static VOMep GetMappedObj(SPListItem item)
        {
            VOMep mepObj = VOMapperMep.GetVOMepFromListItem(item);
            mepObj.Author = (GetSPUser(item, Constants.AuthorField) != null ? GetSPUser(item, Constants.AuthorField).User.Name : string.Empty);
            mepObj.Workflow = SPHelper.GetWorkflowFieldDisplayText(item);
            mepObj.CurrentApprover = SPHelper.GetCurrentApproverName(item);
            mepObj.Entity = GetLookupValuesFromItem(item, Constants.EntityField);
            mepObj.Domaine = GetLookupValuesFromItem(item, Constants.DomainField);
            return mepObj;
        }

        /// <summary>
        /// Get the SPUser object from Item 
        /// </summary>
        /// <param name="item">Item from the list</param>
        /// <param name="fieldName">field name containing the user</param>
        /// <returns>SPUser objectof the specified item</returns>
        private static SPFieldUserValue GetSPUser(SPListItem item, string fieldName)
        {
            SPFieldUserValue fieldValue = null;
            try
            {
                SPSecurity.RunWithElevatedPrivileges(delegate()
                {
                    SPFieldUser field = item.Fields.GetField(fieldName) as SPFieldUser;
                    if (field != null && item[fieldName] != null)
                    {
                        fieldValue = field.GetFieldValue(MappingTool.CatchNull(item[fieldName])) as SPFieldUserValue;
                    }
                });
            }
            catch (SPException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.Message);
            }
            return fieldValue;
        }
        
        /// <summary>
        /// Get item from list in parameter
        /// </summary>
        /// <param name="list">List where item is got</param>
        /// <param name="taskItem">Item from task list</param>
        /// <param name="worflowStatus">worflowStatus</param>
        /// <returns></returns>
        private static SPListItemCollection GetTaskItemsFromList(SPList list, SPListItemCollection taskItems, string worflowStatus, string listPath)
        {
            SPListItemCollection listItems = null;
            List<string> fileNameList = new List<string>();
            foreach (SPListItem taskItem in taskItems)
            {
                SPFieldUrl url = (SPFieldUrl)taskItem.Fields.GetField(Constants.WorkflowLinkField);
                if (url != null && taskItem[Constants.WorkflowLinkField] != null)
                {
                    SPFieldUrlValue fieldValue = url.GetFieldValue(MappingTool.CatchNull(taskItem[Constants.WorkflowLinkField])) as SPFieldUrlValue;
                    if (fieldValue != null)
                    {
                        string fileName = fieldValue.Url.Replace(listPath, string.Empty);
                        fileNameList.Add(fileName);
                    }
                }
            }
            string workflowInternalName = SPHelper.GetInternalFieldName(list, Constants.WorkflowAssociationName);
            if (!string.IsNullOrEmpty(workflowInternalName))
            {
                SPQuery query = QueryBuilder.GetQueryWithFileNames(Constants.WorkflowStatusType, workflowInternalName, worflowStatus, fileNameList);
                listItems = list.GetItems(query);
            }
            return listItems;
        }

        private static List<string> GetLookupValuesFromItem(SPListItem item, string fieldName)
        {
            List<string> entities = new List<string>();
            if (item[fieldName] != null)
            {
                SPFieldLookupValueCollection valuesField = item[fieldName] as SPFieldLookupValueCollection;
                foreach (SPFieldLookupValue value in valuesField)
                {
                    entities.Add(value.LookupValue);
                }
            }
            return entities;
        }
        #endregion
    }
}
