using System;
using System.Runtime.InteropServices;
using System.Linq;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Publishing;
using CreditAgricole.Framework;

namespace CreditAgricole.PageLayout.Features.CreditAgricioleAddPublishingPages
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("17e5c708-b781-4151-8cec-a6c65fcc9bdd")]
    public class CreditAgricioleAddPublishingPagesEventReceiver : SPFeatureReceiver
    {
        /// <summary>
        /// Add a page in pages collection <paramref name="pages"/> using pages layout <paramref name="pageLayout/>. 
        /// </summary>
        /// <remarks>
        /// Only 2 page layouts are considered in the switch case statement.
        /// </remarks>
        /// <param name="pages">Pages collection where page will be added</param>
        /// <param name="pageLayout">Page layout that the page will be based on</param>
        private void AddPageBasedOnPageLayout(PublishingPageCollection pages, Microsoft.SharePoint.Publishing.PageLayout pageLayout)
        {
            switch (pageLayout.Name)
            {
                case Constants.HomePageLayoutName:
                    {
                        PublishingPage newPage = pages.Add(Constants.HomePageName, pageLayout);
                        newPage.ListItem.Update();
                        break;
                    }
                case Constants.ActivitiesPageLayoutName:
                    {
                        PublishingPage newPage = pages.Add(Constants.ActivitiesPageName, pageLayout);
                        newPage.ListItem.Update();
                        break;
                    }
                case Constants.SearchPageLayoutName:
                    {
                        PublishingPage newPage = pages.Add(Constants.SearchPageName, pageLayout);
                        newPage.ListItem.Update();
                        break;
                    }
                default: break;
            }
        }

        /// <summary>
        /// Check weather a page was created using the Page Layout
        /// </summary>
        /// <param name="pages">Publishing pages collection</param>
        /// <param name="pageLayout">Page layout</param>
        /// <returns></returns>
        private bool IsAnyPageBasedOnPageLayout(PublishingPageCollection pages, Microsoft.SharePoint.Publishing.PageLayout pageLayout)
        {
            bool exist = false;
            PublishingPage pageFound = (from page in pages.ToList()
                                        where page.Layout.Name == pageLayout.Name
                                        select page).FirstOrDefault();
            if (pageFound != null)
            {
                exist = true;
            }
            return exist;
        }

        /// <summary>
        /// Create pages in website <paramref name="publishingWeb"/> based on page layouts collection <paramref name="pagelayoutsColl"/>
        /// </summary>
        /// <param name="publishingWeb">website where the pages will be published</param>
        /// <param name="pagelayoutsColl">Page Layouts that the pages will be based on</param>
        private void CreatePublishingPagesByPageLayouts(PublishingWeb publishingWeb, Microsoft.SharePoint.Publishing.PageLayout[] pagelayoutsColl)
        {
            if (publishingWeb != null)
            {
                PublishingPageCollection pages = publishingWeb.GetPublishingPages();
                if (pages != null)
                {
                    if (pagelayoutsColl != null)
                    {
                        foreach (Microsoft.SharePoint.Publishing.PageLayout pageLayout in pagelayoutsColl)
                        {
                            if (IsAnyPageBasedOnPageLayout(pages, pageLayout) == false)
                            {
                                AddPageBasedOnPageLayout(pages, pageLayout);
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Associate content type to Publishing pages list
        /// </summary>
        /// <param name="ctPageLayout">Content type to associate</param>
        /// <param name="pagesList">Publishing pages list</param>
        /// <returns>bool weather the content type is associated to the publishing pages list</returns>
        private bool AssociateContentTypeToPublishingList(SPContentType ctPageLayout, SPList pagesList)
        {
            bool associated = false;
            if (pagesList != null)
            {
                if (ctPageLayout != null)
                {
                    if (pagesList.ContentTypes[ctPageLayout.Name] != null)
                    {
                        associated = true;
                    }
                    else
                    {
                        pagesList.ContentTypes.Add(ctPageLayout);
                        associated = true;
                    }
                }
            }
            return associated;
        }

        // Uncomment the method below to handle the event raised after a feature has been activated.

        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPWeb web = properties.Feature.Parent as SPWeb;
            SPSite site = web.Site;
            SPList pagesList = web.Lists.TryGetList("Pages");
            try
            {
                SPContentType ctPageLayout = site.RootWeb.ContentTypes[Constants.ITBasePageContentType];
                bool associatedCT = AssociateContentTypeToPublishingList(ctPageLayout, pagesList);
                if (associatedCT)
                {
                    PublishingWeb publishingRootWeb = PublishingWeb.GetPublishingWeb(site.RootWeb);
                    if (publishingRootWeb != null)
                    {
                        Microsoft.SharePoint.Publishing.PageLayout[] pagelayoutsColl = publishingRootWeb.GetAvailablePageLayouts(ctPageLayout.Id);
                        PublishingWeb publishingCurrentWeb = PublishingWeb.GetPublishingWeb(web);
                        CreatePublishingPagesByPageLayouts(publishingCurrentWeb, pagelayoutsColl);
                    }
                }
            }
            catch (SPException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.StackTrace);
            }
        }


        // Uncomment the method below to handle the event raised before a feature is deactivated.

        //public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised after a feature has been installed.

        //public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised before a feature is uninstalled.

        //public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        //{
        //}

        // Uncomment the method below to handle the event raised when a feature is upgrading.

        //public override void FeatureUpgrading(SPFeatureReceiverProperties properties, string upgradeActionName, System.Collections.Generic.IDictionary<string, string> parameters)
        //{
        //}
    }
}
