using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Publishing;
using CreditAgricole.Framework;
using System.IO;
using Microsoft.SharePoint.Utilities;

namespace CreditAgricole.PageLayout.Features.CreditAgricolePageLayoutFeature
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("dd37cbf9-21db-48c3-9245-bcaa8749f387")]
    public class CreditAgricolePageLayoutFeatureEventReceiver : SPFeatureReceiver
    {
        // Uncomment the method below to handle the event raised after a feature has been activated.

        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPSite site = properties.Feature.Parent as SPSite;
            SPWeb rootWeb = site.RootWeb;
            string directory = properties.Feature.Definition.RootDirectory;
            directory += @"\ITHomeModule";
            UpdatePageLayoutIfExists(directory, rootWeb, Constants.HomePageLayoutName);
            UpdatePageLayoutIfExists(directory, rootWeb, Constants.ActivitiesPageLayoutName);
            UpdatePageLayoutIfExists(directory, rootWeb, Constants.SearchPageLayoutName);
        }


        private void UpdatePageLayoutIfExists(string directory, SPWeb rootWeb, string pageLayoutName)
        {
            string pageLayoutPath = string.Empty;
            string serverRelativeUrl = SPContext.Current.Site.ServerRelativeUrl;
            try
            {
                if (serverRelativeUrl == "/")
                {
                    pageLayoutPath = string.Format("/_catalogs/masterpage/{0}", pageLayoutName);
                }
                else
                {
                    pageLayoutPath = string.Format("{0}/_catalogs/masterpage/{1}", serverRelativeUrl, pageLayoutName);
                }
                string listName = SPUtility.GetLocalizedString("$Resources:core,Master Page Gallery", "core", rootWeb.Language);
                SPList masterPageList = rootWeb.Lists.TryGetList(listName);
                if (masterPageList != null)
                {
                    SPFolder masterpagesFolder = masterPageList.RootFolder;
                    SPFile pageLayoutFile = rootWeb.GetFile(pageLayoutPath);
                    if (pageLayoutFile != null)
                    {
                        UpdatePageLayout(pageLayoutFile, masterpagesFolder, directory, pageLayoutName);
                    }
                }
            }
            catch (SPException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.StackTrace);
            }
            catch (ArgumentException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.StackTrace);
            }
        }

        private void UpdatePageLayout(SPFile oFile, SPFolder masterpagesFolder, string directory, string oPropertyName)
        {
            try
            {
                if (oFile.CheckOutType != SPFile.SPCheckOutType.None)
                {
                    oFile.CheckIn("auto checkin to update page layout");
                }
                oFile.CheckOut();
                string[] templates = Directory.GetFiles(directory, oPropertyName, System.IO.SearchOption.TopDirectoryOnly);
                string pageLayoutFileName = templates[0];
                FileInfo fileInfo = new FileInfo(pageLayoutFileName);
                byte[] byteArr = System.IO.File.ReadAllBytes(pageLayoutFileName);
                masterpagesFolder.Files.Add(fileInfo.Name, byteArr, true);
                oFile.Update();
                oFile.CheckIn("Check in");
                oFile.Publish("Published");
                oFile.Approve("Approved");
            }
            catch (SPException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.StackTrace);
            }
        }

        // Uncomment the method below to handle the event raised before a feature is deactivated.

        //public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised after a feature has been installed.

        //public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised before a feature is uninstalled.

        //public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        //{
        //}

        // Uncomment the method below to handle the event raised when a feature is upgrading.

        //public override void FeatureUpgrading(SPFeatureReceiverProperties properties, string upgradeActionName, System.Collections.Generic.IDictionary<string, string> parameters)
        //{
        //}
    }
}
